import { View } from '@tarojs/components'
import React, { useCallback, memo, useEffect, useMemo, useRef, useState, ReactNode } from 'react'
import styles from "./index.module.scss"
import classnames from "classnames";
import Taro, { usePullDownRefresh, useRouter, useDidShow } from '@tarojs/taro';
import Popup from '@/components/popup'
import { debounce } from '@/common/util'
import { alert } from '@/common/common'
import { formatPriceDiv, formatDateTime, formatWeightDiv } from '@/common/format'
import { dataLoadingStatus, getFilterData } from '@/common/util'
import Tag from './components/Tag';
import Sort from './components/Sort';
import ChoseCity from './components/ChoseCity';
import Search from '@/components/search'
import ItemLiist from "./components/ItemList"
import InfiniteScroll from '@/components/infiniteScroll'
import { ClientListApi } from '@/api/order'
import { goLink } from '@/common/common'
import SelectGroup from '@/components/SelectGroup'
export default () => {

    const [searchField, setSearchField] = useState<
        {
            page: number;
            size: number;
            label_ids: string,
            purchaser_type: number | string,
            name_phone_or_sale_user: string,
            province_id: any[] | string,
            city_id: any[] | string,
            abstract_sort_key: string,
        }
    >({
        page: 1,
        size: 10,
        label_ids: '',
        purchaser_type: '',
        name_phone_or_sale_user: '',
        province_id: '',
        city_id: '',
        abstract_sort_key: ''
    })

    const [orderData, setOrderData] = useState<{ list: any[]; total: number }>({ list: [], total: 0 })
    const { fetchData: listFetchData, state: orderState } = ClientListApi()
    const getOrderList = async () => {
        let res = await listFetchData({
            ...getFilterData(searchField)
        })
        setOrderData((e) => ({ ...e, list: res.data?.list, total: res.data?.total }))
        setRefresherTriggeredStatus(() => false)
    }

    //监听筛选条件变化
    useEffect(() => {
        getOrderList()
    }, [searchField])


    //输入了搜索关键字
    const getSearchData = useCallback((e) => {
        pageNum.current.page = 1
        setOrderData(() => ({ list: [], total: 0 }))
        setSearchField((val) => ({ ...val, name_phone_or_sale_user: e, size: 10 }))
    }, [])

    //数据加载状态
    const statusMore = useMemo(() => {
        return dataLoadingStatus({ list: orderData.list, total: orderData.total, status: orderState.loading! })
    }, [orderData, orderState])

    //上拉加载数据
    const pageNum = useRef({ size: searchField.size, page: searchField.page })
    const getScrolltolower = useCallback(() => {
        if (orderData.list.length < orderData.total) {
            pageNum.current.page++
            const size = pageNum.current.size * pageNum.current.page
            setSearchField({ ...searchField, size })
        }
    }, [orderData])

    //列表下拉刷新
    const [refresherTriggeredStatus, setRefresherTriggeredStatus] = useState(false)
    const getRefresherRefresh = async () => {
        pageNum.current.size = 1
        setRefresherTriggeredStatus(true)
        setSearchField((val) => ({ ...val, size: 10 }))
    }

    //选择排序
    const sortRefRef = useRef<any>()
    const handSort = useCallback((e: string) => {
        pageNum.current.page = 1
        setSearchField((val) => ({ ...val, size: 10, abstract_sort_key: e }))
        sortRefRef.current = e
    }, [])

    const SortRef = useRef<any>()
    const TagRef = useRef<any>()

    //筛选城市
    const ChoseCityRef = useRef<any>()
    const handCity = useCallback((provinceVal, cityVal) => {
        const provinceArr: number[] = []
        const cityArr: number[] = []
        provinceVal.length && provinceVal.forEach(item => {
            provinceArr.push(item.id)
        })
        cityVal.length && cityVal.forEach(item => {
            cityArr.push(item.id)
        })
        pageNum.current.page = 1
        let provinceRes = provinceArr?.toString()
        let cityRes = cityArr?.toString()
        console.log(provinceRes, cityRes)
        setSearchField((val) => ({ ...val, province_id: provinceArr.length > 0 ? provinceRes : '', city_id: cityArr.length > 0 ? cityRes : '', size: 10 }))
        // ChoseCityRef.current.close()
    }, [])

    //选择客户类型
    const handType = useCallback((val) => {
        const arr = val.filter(item => { return item.checked })
        pageNum.current.page = 1
        setSearchField((val) => ({ ...val, size: 10, purchaser_type: arr[0]?.id }))
    }, [])

    //选择标签
    const handTags = useCallback((val) => {
        const arr: any[] = []
        const arrOne = val.filter(item => { return item.checked })
        arrOne.forEach(item => {
            arr.push(item.id)
        })
        pageNum.current.page = 1
        let res = arr.toString()
        setSearchField((val) => ({ ...val, size: 10, label_ids: arr.length === 0 ? '' : res }))
    }, [])


    return (
        <View className={styles.mainBox}>
            <View className={styles.topBox}>
                <View className={styles.search_input}>
                    <Search placeholder='搜索客户名称、电话、业务员' showBtn={false} changeOnSearch={getSearchData} debounceTime={300} />
                </View>
                <View className={styles.menuBox} >
                    <SelectGroup>
                        <Sort ref={SortRef} handSort={handSort} onCloseOverlay={() => true}></Sort>
                        <ChoseCity ref={ChoseCityRef} handCity={handCity} onCloseOverlay={() => true}></ChoseCity>
                        <Tag ref={TagRef} handTags={handTags} handType={handType} onCloseOverlay={() => true}></Tag>
                    </SelectGroup>
                </View>
            </View>
            <View className={styles.totalFont}>共 {orderData?.total || 0} 个客户</View>
            <View className={styles.order_list}>
                <InfiniteScroll
                    statusMore={statusMore}
                    selfonScrollToLower={getScrolltolower}
                    refresherEnabled={true}
                    refresherTriggered={refresherTriggeredStatus}
                    selfOnRefresherRefresh={getRefresherRefresh}>
                    {orderData?.list?.map((item, index) => {
                        return (
                            <ItemLiist sortId={sortRefRef.current} obj={item}></ItemLiist>
                        )
                    })}
                </InfiniteScroll>
            </View>
            <View className={styles.bottom_box}>
                <View className={styles.bottom_btn} onClick={() => goLink('/pages/customerEditor/index?type=add')}>新建客户</View>
            </View>
        </View >
    )
}
